import json
import os
import sys
import wave
import pyaudio
import Queue
import struct
import urllib2
import requests
from tempfile import mkstemp
from statistics import median
from scikits.samplerate import resample
from scikits.audiolab import Sndfile, Format, wavread
from pocketsphinx import Decoder
from vocabcompiler import compile

have_sphinx_dictionary = False
RATE = 16000
power_levels = []

def best_speech_result(pyaudio, audio_data, profile, stt_type = "google"):
  wav_name = data_to_wav(pyaudio, audio_data)
  if wav_name == "":
    print("error saving data")
    return ""
  if stt_type == 'google':
    output = best_google_speech_result(pyaudio, wav_name, profile)
  elif stt_type == 'att':
    output = best_att_speech_result(pyaudio, wav_name, profile)
  elif stt_type == 'sphinx':
    output = best_sphinx_speech_result(pyaudio, wav_name, profile)

  os.remove(wav_name)
  return output

def best_sphinx_speech_result(pyaudio, wav_name, profile):
  if not have_sphinx_dictionary:
    if not profile.has_key("words"):
      raise(KeyError("Pass the possible words in in profile"))
    compile("sentences.txt", "dictionary.dic", "language_model.lm", profile["words"])
    global have_sphinx_dictionary
    have_sphinx_dictionary = True

  wav_file = file(wav_name, 'rb')
  speechRec = Decoder(
    hmm  = "/usr/local/share/pocketsphinx/model/hmm/en_US/hub4wsj_sc_8k",
    lm   = "language_model.lm",
    dict = "dictionary.dic"
  )

  speechRec.decode_raw(wav_file)
  results = speechRec.get_hyp()
  return results[0]

def best_google_speech_result(pyaudio, wav_name, profile):
  if not profile.has_key("key") or profile["key"] == '':
    profile["key"] = "AIzaSyBOti4mM-6x9WDnZIjIeyEU21OpBXqWBgw"
  flac_file = wav_to_flac(wav_name)
  return best_google_result(flac_to_google_result(flac_file, profile["key"]))

def best_google_speech_result(pyaudio, wav_name, profile):
  if not profile.has_key("key") or profile["key"] == '':
    profile["key"] = "AIzaSyBOti4mM-6x9WDnZIjIeyEU21OpBXqWBgw"
  flac_file = wav_to_flac(wav_name)
  return best_google_result(flac_to_google_result(flac_file, profile["key"]))
def best_att_speech_result(pyaudio, wav_name, profile):
  if not profile.has_key("ATT_TOKEN"):
    if not profile.has_key("ATT_APP_KEY") or profile["ATT_APP_KEY"] == "":
      raise(KeyError("Pass in ATT_APP_KEY in profile"))
    if not profile.has_key("ATT_APP_SECRET") or profile["ATT_APP_SECRET"] == "":
      raise(KeyError("Pass in ATT_APP_SECRET in profile"))

    # Get Access Token via OAuth.
    # https://matrix.bf.sl.attcompute.com/apps/constellation-sandbox
    response = requests.post("https://api.att.com/oauth/token", {
        "client_id": profile["ATT_APP_KEY"],
        "client_secret": profile["ATT_APP_SECRET"],
        "grant_type": "client_credentials",
        "scope": "SPEECH"
    })
    content = json.loads(response.content)
    profile["ATT_TOKEN"] = content["access_token"]

  response = requests.post("https://api.att.com/speech/v3/speechToText",
          headers = {"Authorization": "Bearer %s" % profile["ATT_TOKEN"],
                     "Accept": "application/json",
                     "Content-Type": "audio/wav",
                     "X-SpeechContext": "Generic",
                    }, data = open(wav_name, 'rb'))
  return json.loads(response.content)['Recognition']['NBest'][0]['ResultText']

def put_audio_data_in_queue(p, queue):
  CHUNK = 4092 #Takes approximately 1/4 second at RATE = 16000

  quiet_time = 0
  current_data = ""

  stream = p.open(input_device_index = None, rate = RATE, channels = 1,
                  frames_per_buffer = CHUNK, format = pyaudio.paInt16,
                  input = True)
  while True:
    data = stream.read(CHUNK)

    # 2 bytes per paInt16
    levels = struct.unpack("%dh"%(len(data)/2), data)

    sum_squares = 0
    for level in levels:
        sum_squares += level * level
    power_levels.append(sum_squares / CHUNK)

    # Average median power over last half minute
    # With a factor of 10 to reject noise
    thresh = median(power_levels[len(power_levels) - 20:]) * 10

    if (sum_squares / CHUNK > thresh):
      current_data = current_data + data
      quiet_time = 0
    else:
      if (quiet_time > 0.5):               #0.5 second maximum of quietness
        if (len(current_data) / RATE > 1): #0.5 second minimum of loudness
          queue.put(current_data)
        current_data = ""
      quiet_time += (CHUNK + 0.0) / RATE

  stream.stop_stream()
  stream.close()

def data_to_wav(p, audio_data):
  cd, wav_name = mkstemp('tmp.wav')

  wav_file = wave.open(wav_name, 'wb')
  wav_file.setnchannels(1)
  wav_file.setsampwidth(p.get_sample_size(pyaudio.paInt16))
  wav_file.setframerate(RATE)
  wav_file.writeframes(audio_data)
  wav_file.close()

  return wav_name

def wav_to_flac(wav_name):
  cd, tmp_name = mkstemp('tmp.flac')

  Signal, fs = wavread(wav_name)[:2]
  assert(fs == RATE)

  fmt = Format('flac', 'pcm16')
  nchannels = 1
  flac_file = Sndfile(tmp_name, 'w', fmt, nchannels, RATE)
  flac_file.write_frames(Signal)

  return tmp_name

def best_google_result(result):
  if result == "":
    print("No result")
    return ""
  try:
    lines = result.splitlines()
    if "transcript" in lines[0] or len(lines[0]) > 15:
      print("\nFirst line has info:\n" + lines[0] + '\n')
    else:
      return json.loads(lines[1])['result'][0]['alternative'][0]['transcript']
  except:
    return "Error"

def flac_to_google_result(flac_name, key):
  if flac_name == "":
    return ""

  flac_file = open(flac_name, 'rb')

  url = "https://www.google.com/speech-api/v2/recognize"
  url += "?output=json&lang=en-us&key=" + key

  header = {'Content-Type' : 'audio/x-flac; rate=16000'}
  req = urllib2.Request(url, data = flac_file.read(), headers = header)
  result = urllib2.urlopen(req)

  flac_file.close()
  os.remove(flac_name)

  return result.read()

if __name__ == "__main__":
  from thread import start_new_thread
  p = pyaudio.PyAudio()
  audio_data_queue = Queue.Queue()
  start_new_thread(put_audio_data_in_queue, (p, audio_data_queue,))
  while True:
    data = audio_data_queue.get()
    print(len(data))
    print(best_speech_result(p, data, {}, "google"))



import os
import sys

import g2p


def text2lm(in_filename, out_filename):
    """Wrapper around the language model compilation tools"""
    def text2idngram(in_filename, out_filename):
        cmd = "text2idngram -vocab %s < %s -idngram temp.idngram" % (out_filename,
                                                                     in_filename)
        os.system(cmd)

    def idngram2lm(in_filename, out_filename):
        cmd = "idngram2lm -idngram temp.idngram -vocab %s -arpa %s" % (
            in_filename, out_filename)
        os.system(cmd)

    text2idngram(in_filename, in_filename)
    idngram2lm(in_filename, out_filename)


def compile(sentences, dictionary, languagemodel, words):

    # create the dictionary
    pronounced = g2p.translateWords(words)
    zipped = zip(words, pronounced)
    lines = ["%s %s" % (x, y) for x, y in zipped]

    with open(dictionary, "w") as f:
        f.write("\n".join(lines) + "\n")

    # create the language model
    with open(sentences, "w") as f:
        f.write("\n".join(words) + "\n")
        f.write("<s> \n </s> \n")
        f.close()

    # make language model
    text2lm(sentences, languagemodel)



from pyspeech import best_speech_result
import unittest
from pyaudio import PyAudio
import Queue

class PyspeechTest(unittest.TestCase):
  def setUp(self):
    self.p = PyAudio()

  def test_google_stt(self):
    good_morning = open('example_wavs/good_morning.wav', 'rb')
    output = best_speech_result(self.p, good_morning.read(), {}, "google")
    self.assertEqual(output, "good morning")

    hello_world = open('example_wavs/hello_world.wav', 'rb')
    output = best_speech_result(self.p, hello_world.read(), {}, "google")
    self.assertEqual(output, "hello world")

  # This will fail without a valid wit_token in profile.yml
  def test_wit_stt(self):
    import yaml
    profile = yaml.load(open("profile.yml").read())
    good_morning = open('example_wavs/good_morning.wav', 'rb')
    output = best_speech_result(self.p, good_morning.read(), profile, "wit")
    self.assertEqual(output, "good morning")

    hello_world = open('example_wavs/hello_world.wav', 'rb')
    output = best_speech_result(self.p, hello_world.read(), profile, "wit")
    self.assertEqual(output, "hello world")

if __name__ == "__main__":
  unittest.main()



from thread import start_new_thread
from pyaudio import PyAudio
from pyspeech import best_speech_result, put_audio_data_in_queue
from time import sleep
import Queue

def background_stt(queue, profile, stt_type = 'google'):
  start_new_thread(_spawn_listeners, (queue, profile, stt_type,))

def _spawn_listeners(queue, profile, stt_type):
  p = PyAudio()
  audio_data_queue = Queue.Queue()
  start_new_thread(put_audio_data_in_queue, (p, audio_data_queue,))
  while True:
    _listen(p, queue, audio_data_queue, profile, stt_type)

def _listen(pyaudio, queue_out, queue_in, profile, stt_type):
  output = best_speech_result(pyaudio, queue_in.get(), profile, stt_type)
  if output != "":
    queue_out.put(output)

if __name__ == "__main__":
  import yaml
  profile = yaml.load(open("profile.yml").read())
  q = Queue.Queue()
  background_stt(q, profile, 'att')

  while True:
    print(q.get())




import os
import subprocess
import re

TEMP_FILENAME = "g2ptemp"
PHONE_MATCH = re.compile(r'<s> (.*) </s>')
PHONETISAURUS_PATH = os.environ['HOME'] + "/Dcuments/phonetisaurus"


def parseLine(line):
    return PHONE_MATCH.search(line).group(1)


def parseOutput(output):
    return PHONE_MATCH.findall(output)


def translateWord(word):
    out = subprocess.check_output(['phonetisaurus-g2p', '--model=%s' %
                                  PHONETISAURUS_PATH + "/g014b2b.fst", '--input=%s' % word])
    return parseLine(out)


def translateWords(words):
    full_text = '\n'.join(words)

    f = open(TEMP_FILENAME, "wb")
    f.write(full_text)
    f.flush()

    output = translateFile(TEMP_FILENAME)
    os.remove(TEMP_FILENAME)

    return output


def translateFile(input_filename, output_filename=None):
    out = subprocess.check_output(['phonetisaurus-g2p', '--model=%s' %
        PHONETISAURUS_PATH + "/g014b2b.fst", '--input=%s' % input_filename, '--words', '--isfile'])
    out = parseOutput(out)

    if output_filename:
        out = '\n'.join(out)

        f = open(output_filename, "wb")
        f.write(out)
        f.close()

        return None

    return out

if __name__ == "__main__":

    translateFile(PHONETISAURUS_PATH + "/phonetisaurus/sentences.txt",
                  PHONETISAURUS_PATH + "/phonetisaurus/dictionary.dic")



#!/usr/bin/env python
from setuptools import setup

setup(name='constant_listener',
      version='0.0.4',
      author='Matt Wismer',
      author_email='mattwis86@gmail.com',
      description='STT engine that listens in the background',
      license='MIT',
      packages=['constant_listener'],
      package_dir={'constant_listener': 'constant_listener'},
      url='https://github.com/MattWis/constant_listener.git',
      install_requires=['scikits.audiolab', 'scikits.samplerate', 'pyaudio'])
